import scipy as sp
import datetime as dt
import holidays
import csv
import random
# import itertools
import os
import os.path
import subprocess
import zipfile
import locale
import calendar as cal

locale = locale.setlocale(locale.LC_ALL, 'de_DE.UTF8')


def diff_full_months(start, end):

    start = start
    end = end + dt.timedelta(days=1)

    full_months = (end.year - start.year) * 12 + end.month - start.month

    if start.day > end.day:
        full_months = full_months - 1

    return full_months


def diff_months(start, end):

    months = (end.year - start.year) * 12 + end.month - start.month + 1

    return months


def diff_weeks(start, end):

    weeks = ((end + dt.timedelta(days=1)) - start).days / 7

    return weeks


def get_calendar_week(date):

    calendar_week = date.isocalendar()[1]

    return int(calendar_week)


def float2time(float_time):

    time = '{0:02.0f}.{1:02.0f}'.format(*divmod(round(float_time * 60), 60))

    return time


def float2duration(float_time):

    time = '{0:02.0f}:{1:02.0f}'.format(*divmod(round(float(float_time) * 60), 60))

    return time


def check_day_hours(list_of_workinghours, workingday_list_position, timestep):

    if list_of_workinghours[workingday_list_position][7] + timestep <= 6:
        return True
    else:
        return False


def check_week_hours(list_of_workinghours, workingday_list_position,
                     timestep, list_of_filled_workinghours):

    current_calendar_week = get_calendar_week(list_of_workinghours[workingday_list_position][0])
    list_of_affected_days = [same_calendar_week for same_calendar_week in list_of_workinghours if get_calendar_week(same_calendar_week[0]) == current_calendar_week]
    list_of_other_affected_days = [same_calendar_week for same_calendar_week in list_of_filled_workinghours if get_calendar_week(same_calendar_week[0]) == current_calendar_week]

    for a in range(len(list_of_other_affected_days)):
        list_of_affected_days.append(list_of_other_affected_days[a])

    week_sum = 0
    for x in range(len(list_of_affected_days)):
        week_sum = week_sum + list_of_affected_days[x][7]

    if week_sum + timestep <= 19:
        return True
    else:
        return False

def move_filled_day(list_of_workinghours, workingday_list_position, list_of_filled_workinghours):

    list_of_filled_workinghours.append(list_of_workinghours[workingday_list_position])
    list_of_workinghours = [unfilled_day for unfilled_day in list_of_workinghours if unfilled_day not in list_of_filled_workinghours]

    return list_of_workinghours, list_of_filled_workinghours
    print("Day " + list_of_workinghours[workingday_list_position][0].strftime("%Y-%m-%d") + " full and moved.")

def move_filled_week(list_of_workinghours, workingday_list_position, list_of_filled_workinghours):

    current_calendar_week = get_calendar_week(list_of_workinghours[workingday_list_position][0])
    list_of_affected_days = [same_calendar_week for same_calendar_week in list_of_workinghours if get_calendar_week(same_calendar_week[0]) == current_calendar_week]

    for x in range(len(list_of_affected_days)):
        list_of_filled_workinghours.append(list_of_affected_days[x])
    list_of_workinghours = [unfilled_day for unfilled_day in list_of_workinghours if unfilled_day not in list_of_filled_workinghours]

    return list_of_workinghours, list_of_filled_workinghours
    print("Week " + str(current_calendar_week) + " full.")


def make_up_workinghours(month, start, end, workhours_per_week, workdays_per_week, already_worked = 0):

    full_months         = diff_full_months(start, end)
    months              = diff_months(start, end)
    weeks               = diff_weeks(start, end)
    days                = (end - start).days + 1
    workdays_total      = int(workdays_per_week * weeks)
    workhours_total     = int(workhours_per_week * weeks)
    workhours_per_month = int(workhours_per_week * weeks / months)
    holidayhours_total  = sp.floor(1 / 3 * workhours_per_week * full_months)
    holidaydays_total   = sp.ceil(1 / 3 * workdays_per_week * full_months)

    print("")
    print("Anzahl ganzer Monate:        ", full_months)
    print("Anzahl Monate:               ", round(months,2))
    print("Anzahl Wochen:               ", round(weeks,2))
    print("Anzahl Tagen:                ", int(days))
    print("Arbeitstage (Gesamt):        ", workdays_total)
    print("Arbeitsstunden (Gesamt):     ", workhours_total)
    print("Arbeitsstunden pro Monat (Ø):", workhours_per_month)
    print("Urlaubsanspruch in Stunden:  ", holidayhours_total)
    print("Urlaubstage:                 ", holidaydays_total)
    print("")

    full_date_list = [(start + dt.timedelta( days = x )).date() for x in range(0, days)]
    holiday_list   = holidays.Holidays(start.year, 'BY').get_holiday_list()[0] + holidays.Holidays(end.year, 'BY').get_holiday_list()[0]

    global timestep
    global start_time_earliest
    global start_time_latest
    global preferred_weekdays
    global fixed_times_weekdays
    global exclude_list

    timestep = 0.5
    start_time_earliest = 9
    start_time_latest = 12
    preferred_weekdays = [0,1,2,3,4]
    preset_list = [[],[]]
    exclude_list = []

    if advanced_settings:

        timestep = input(
'''Working hours are distributed over random days
by adding a certain amount of time to them.
What amount of time would you like to add per step?
Possible values are displayed below.

     15 minutes
     20 minutes
     30 minutes (default)
     40 minutes
     45 minutes
     60 minutes
    120 minutes

> '''
        )

        if timestep != "":
            timestep = int(timestep) / 60
        else:
            timestep = 0.5

        start_time_earliest = input(
'''Early bird? Night owl? No worries, I got you covered...
What time would you like to start the earliest?
Must be integer [0:23]. Defaults to 9.

> '''
        )

        if start_time_earliest != "":
            start_time_earliest = int(start_time_earliest)
        else:
            start_time_earliest = 9

        start_time_latest = input(
'''What time would you like to start the latest?
Must be integer [0:23] >= earliest start time. Defaults to 12.

> '''
        )

        if start_time_latest != "":
            start_time_latest = int(start_time_latest)
        else:
            start_time_latest = 12

        if start_time_latest <= start_time_earliest:
            print("Lastest start time before earliest start time. I will continue with earliest start time + 2.")
            start_time_latest = start_time_earliest + 2

        preferred_weekdays = input(
'''Which weekdays do you prefer to work on?
Must be comma separated list of integers [0:5]. Defaults to [0,1,2,3,4].

    0 = Monday
    1 = Tuesday
    2 = Wednesday
    3 = Thursday
    4 = Friday
    5 = Saturday

Example Input: 0, 1, 2
Example Meaning: Your preferred weekdays are Monday, Tuesday and Wednesday.

> '''
        )

        if preferred_weekdays != "":
            preferred_weekdays = [int(x) for x in preferred_weekdays.replace(" ","").split(",")]
        else:
            preferred_weekdays = [0,1,2,3,4]


        if len(preferred_weekdays) < workdays_per_week:
            print("Not enough preferred weekdays given. I will continue with default.")
            preferred_weekdays = [0,1,2,3,4]

        preset_list = input(
'''Are there any specific dates you want to preset the amount of hours you worked?
Must be comma separated list of hours mapped to dates. Defaults to [].

Example Input: 3 => 12.01.2019, 5 => 13.01.2019
Example Meaning: You worked 3 hours on 12.01.2019 and 5 hours on 13.01.2019.

> '''
        )

        if len(preferred_weekdays) < workdays_per_week:
            print("Not enough preferred weekdays given. I will continue with default.")
            preferred_weekdays = [0,1,2,3,4]

        preset_list = input(
'''Are there any specific dates you want to preset the amount of hours you worked?
Must be comma separated list of hours mapped to dates. Defaults to [].

Example Input: 3 => 12.01.2019, 5 => 13.01.2019
Example Meaning: You worked 3 hours on 12.01.2019 and 5 hours on 13.01.2019.

> '''
        )



        fixed_times_weekdays = input(
'''Which weekdays do you prefer to work on?
Must be comma separated list of touples of integers [0:5] (days) and times hh-hh separeted by "=>". Defaults to [].

    0 = Monday
    1 = Tuesday
    2 = Wednesday
    3 = Thursday
    4 = Friday
    5 = Saturday

Example Input: 3 => 13-16
Example Meaning: You always work thursday from 13:00 to 16:00

> '''
        )

        if fixed_times_weekdays != "":
            # split the days
            fixed_times_weekdays = [x for x in fixed_times_weekdays.replace(" ","").split(",")]
            day_time_dict = dict()
            for day_time_touple in fixed_times_weekdays:
                day, times = day_time_touple.split("=>")

                day = int(day)

                times = times.split("-")
                times = [int(t) for t in times]
                tstart = min(times)
                tlength = max(times)-tstart
                if day in day_time_dict:
                    print("Day {} more than once in list, ignoring all occurances except the first one.".format(day))
                else:
                    day_time_dict[day] = [tstart, tlength]
            fixed_times_weekdays = day_time_dict

        else:
            fixed_times_weekdays = dict()


        if preset_list != "":
            preset_duration = [float(x.split("=>")[0]) for x in preset_list.replace(" ","").split(",")]
            preset_workingdays = [dt.datetime.strptime(x.split("=>")[1], "%d.%m.%Y").date() for x in preset_list.replace(" ","").split(",")]
            preset_list = [preset_workingdays, preset_duration]
        else:
            preset_list = [[],[]]

        exclude_list = input(
'''Are there any specific dates you want to exclude from the list of working days?
Must be space separated list of dates. Defaults to [].

Example Input: 12.01.2019, 13.01.2019, 14.01.2019
Example Meaning: Those three dates are treated like public holidays.

> '''
        )

        if exclude_list != "":
            exclude_list = [dt.datetime.strptime(x, "%d.%m.%Y").date() for x in exclude_list.replace(" ","").split(",")]
        else:
            exclude_list = []
        
    for retry in range(10000):
        
        if (workhours_per_week <= 19 and workdays_per_week <= 5):
            date_list      = [day for day in full_date_list if day.weekday() in preferred_weekdays]
            date_list      = [day for day in date_list if day not in holiday_list]
            date_list      = [day for day in date_list if day not in exclude_list]
        elif (workhours_per_week <= 19):
            print("Too many workdays per week given!")
            print("")
            break
        else:
            print("Too many workhours per week given!")
            print("")
            break

        made_up_workinghours = []
        make_up_workinghours = []

        for i in range(len(date_list)):

            workingday         = date_list[i]

            make_up_duration   = 0.0
            used_workinghours = 0

            if workingday.weekday() in fixed_times_weekdays:
                tstart, tlength = fixed_times_weekdays[workingday.weekday()]
                make_up_start_time = tstart
                make_up_end_time   = random.randint(tstart+tlength,20)
                make_up_duration = tlength
                used_workinghours += tlength
            else:
                make_up_start_time = random.randint(start_time_earliest,start_time_latest)
                make_up_end_time   = random.randint(16,20)

            if workingday in preset_list[0]:
                ind = preset_list[0].index(workingday)
                make_up_duration = preset_list[1][ind]

            make_up_break      = sp.random.choice([0, 0.25, 0.5, 0.75, 1.0], p=[0.8, 0.025, 0.075, 0.025, 0.075])

            make_up_workinghours.append([workingday])
            make_up_workinghours[i].append(workingday.strftime("%d.%m.%Y"))
            make_up_workinghours[i].append(cal.day_abbr[workingday.weekday()])
            make_up_workinghours[i].append(make_up_start_time)
            make_up_workinghours[i].append(float2time(make_up_start_time))
            make_up_workinghours[i].append(make_up_end_time)
            make_up_workinghours[i].append(float2time(make_up_end_time))
            make_up_workinghours[i].append(make_up_duration)
            make_up_workinghours[i].append(float2duration(make_up_duration))
            make_up_workinghours[i].append(make_up_break)
            make_up_workinghours[i].append(float2duration(make_up_break))
            make_up_workinghours[i].append('')
        for preset_workingday in preset_list[0]:

            preset_workingday = [workingday[0] for workingday in make_up_workinghours].index(preset_workingday)
            make_up_workinghours, made_up_workinghours = move_filled_day(make_up_workinghours, preset_workingday, made_up_workinghours)

        used_workinghours += sum(preset_list[1])

        while (used_workinghours < workhours_total):

            if (used_workinghours + timestep > workhours_total):
                timestep = timestep / 2

            if (len(make_up_workinghours) == 0):
                break

            random_workingday = random.randint( 0, len(make_up_workinghours) - 1 )
            check_week = check_week_hours(make_up_workinghours, random_workingday, timestep, made_up_workinghours)
            check_day  = check_day_hours(make_up_workinghours, random_workingday, timestep)

            if (check_week == True and check_day == True):

                make_up_duration = make_up_workinghours[random_workingday][7] + timestep

                make_up_workinghours[random_workingday][7] = make_up_duration
                make_up_workinghours[random_workingday][8] = float2duration(make_up_duration)

                used_workinghours = used_workinghours + timestep

            elif (check_week == True and check_day == False):

                make_up_workinghours, made_up_workinghours = move_filled_day(make_up_workinghours, random_workingday, made_up_workinghours)

            else:

                make_up_workinghours, made_up_workinghours = move_filled_week(make_up_workinghours, random_workingday, made_up_workinghours)

            if (used_workinghours == workhours_total):

                remaining_filled_days = [filled_day for filled_day in make_up_workinghours if filled_day[7] != 0]

                for j in range(len(remaining_filled_days)):

                    made_up_workinghours.append(remaining_filled_days[j])

                break

        for i in range(len(made_up_workinghours)):

            made_up_start_time = made_up_workinghours[i][3]
            made_up_duration   = made_up_workinghours[i][7]
            made_up_break      = made_up_workinghours[i][9]

            made_up_end_time = made_up_start_time + made_up_duration + made_up_break

            made_up_workinghours[i][5] = made_up_end_time
            made_up_workinghours[i][6] = float2time(made_up_end_time)

        used_holidayhours = 0

        while (used_holidayhours < holidayhours_total):

            random_holidayday = random.randint( 0, len(made_up_workinghours) - 1 )

            if (made_up_workinghours[random_holidayday][7] >= holidayhours_total / holidaydays_total - 0.25) and (made_up_workinghours[random_holidayday][9] == 0):

                made_up_workinghours[random_holidayday][11] = "Urlaub"
                used_holidayhours = used_holidayhours + made_up_workinghours[random_holidayday][7]

        if (used_workinghours == workhours_total and used_holidayhours == holidayhours_total):

            break

    if retry == 0:
        print(
'''I finished making up working hours for you.
You can find the data in the output-folder.'''
        )
        print("")
    elif retry + 1 < 10000:
        print(
'''I ran into some trouble making up working hours.
I retried ''' + str(retry + 1) + ''' times, but now I finished.
You can find the data in the output-folder.'''
        )
        print("")
    elif retry + 1 >= 10000:
        print(
'''I ran into some trouble making up working hours.
I retried ''' + str(retry + 1) + ''' times and couldn't do it!
I have no idea what happend, please restart me!'''
        )
        print("")

    return made_up_workinghours


# User Input Vertragsdaten

print("")
start_date = input(
'''What is the start date of your contract? (dd.mm.yyyy)
> '''
)
end_date   = input(
'''What is the end date of your contract? (dd.mm.yyyy)
> '''
)

start = dt.datetime.strptime(start_date, "%d.%m.%Y")
end   = dt.datetime.strptime(end_date, "%d.%m.%Y")

print("")
workhours_per_week = float(input(
'''How many hours do you have to work per week?
> '''
))
workdays_per_week  = float(input(
'''How many days do you have to work per week?
> '''
))

# User Input Persönliche Daten

print("")
name          = input(
'''What is your last name?
> '''
)
firstname     = input(
'''What is your first name(s)?
> '''
)
dateofbirth   = input(
'''What is your date of birth? (dd.mm.yyyy)
> '''
)

# User Input Institut

institute = ''
street = ''
city = ''
postcode = ''
faculty = ''

print("")
instituteabbr = input(
'''Please specify your institute?
You may use one of the following shortcodes:

    physics:  Department Physik
    astro:    Astronomisches Institut
    biophy:   Lehrstuhl für Biophysik
    didaktik: Professur für Didaktik der Physik
    ecap:     Erlangen Centre for Astroparticle Physics
    fkp:      Lehrstuhl für Festköperphysik
    kristall: Lehrstuhl für Kristallographie und Strukturphysik
    lap:      Lehrstuhl für Angewandt Physik
    laser:    Lehrstuhl für Laserphysik
    mpl:      Max-Planck-Institut für die Physik des Lichts
    supra:    Lehrstuhl für Experimentalphysik (Supraleitung)
    theo1:    Lehrstuhl für Theoretische Physik 1
    theo2:    Lehrstuhl für Theoretische Physik 2
    theo3:    Lehrstuhl für Theoretische Physik 3 (Quantengravitation)
    theo4:    Lehrstuhl für Theoretische Festkörperphysik

    cs:       Department Informatik
    cs1:      Lehrstuhl Informatik 1
    cs2:      Lehrstuhl Informatik 2
    ...
    cs14:     Lehrstuhl Informatik 14

> '''
)

list_of_implemented_institutes = [f for f in os.listdir("../input/institutes/") if os.path.isfile(os.path.join("../input/institutes/", f))]

if instituteabbr in list_of_implemented_institutes:
    with open('../input/institutes/' + instituteabbr, 'r') as institutefile:
        instituteinfo = institutefile.read()
    exec(instituteinfo)
else:
    print("")
    print("Your institute is not registered yet, please provide me with some details.")
    institute     = input(
'''Name of your institute?
> '''
    )
    street        = input(
'''Street of your institute?
> '''
    )
    city          = input(
'''City of your institute?
> '''
    )
    postcode      = input(
'''Post code of your institute?
> '''
    )
    faculty       = input(
'''Which faculty does your institute belong to?
Currently implemented are:

    fau:       Friedrich-Alexander-University
    philtheo:  Faculty of Humanities, Social Sciences and Theology
    recht:     Faculty of Law
    wirt:      Faculty of Business and Economics
    rechtwirt: Faculty of Business, Economics and Law
    med:       Faculty of Medicine
    nat:       Faculty of Sciences
    tech:      Faculty of Engeneering

> '''
    )

    wanna_give_me_a_logo = input(
'''Do you want to specify a file for your own logo? (yes/no)
> ''')
    if (wanna_give_me_a_logo == "y" or wanna_give_me_a_logo == "yes"):
        wanna_give_me_a_logo = True
    else:
        wanna_give_me_a_logo = False

    if wanna_give_me_a_logo:
        logo = input(
'''Where can I find the logo-file?
> '''
        )
    else:
        print("I will proceed without an institute logo.")

# User Input Arbeitsstunden

print("")
filename = input(
'''Where can I find the csv-file containing working hours?
The file must be written in the following format:
    
    # iso-date , start , end   , duration , break , comment
    2018-02-01 , 10:00 , 13:00 , 3:00     , 0:30  , 
    2018-02-02 , 10:00 , 12:00 , 2:00     ,       , 
    2018-02-05 , 10:00 , 11:00 , 1:00     ,       , 
    2018-02-06 , 10:00 , 13:00 , 3:00     ,       , 
    2018-02-08 , 10:00 , 13:00 , 3:00     ,       , Urlaub

Header must start with number sign # to mark it as a comment!
If left empty I can still generate a preview.

> '''
)


# Überprüfung Arbeitszeiten

filename = ''

if filename == '':
    
    print("")
    should_i_make_up_workinghours = input(
'''Do you want me to generate a preview of my capabilities? (yes/no)

> '''
    )
    if (should_i_make_up_workinghours == "y" or should_i_make_up_workinghours == "yes"):
        should_i_make_up_workinghours = True

    print("")
    advanced_settings = input(
'''Do you want to use advanced settings? (yes/no)

> '''
    )
    if (advanced_settings == "y" or advanced_settings == "yes"):
        advanced_settings = True
    else:
        advanced_settings = False 

    if should_i_make_up_workinghours:
        
        should_i_do_it_again = True
        
        while should_i_do_it_again:
        
            workinghours = []
        
            made_up = make_up_workinghours(2, start, end, workhours_per_week, workdays_per_week)
            workinghours.append(made_up)

            for m in range(int(sp.ceil(diff_months(start,end)))):

                month = (start.month + m - 1) % 12 + 1
                monthname = cal.month_name[month]

                print(month)

                entries = [day for day in workinghours[0] if day[0].month == month]
                entries.sort()

                year = entries[0][0].year

                total = 0
                for x in range(len(entries)):
                    total = total + entries[x][7]

                filename = name.lower() + '-' + firstname.lower() + '_' + str(year) + '-' + '{0:0>2}'.format(month) + '_arbeitsstunden'

                with open(filename + '.csv', 'w') as csvfile:
                    file = csv.writer(csvfile)
                    for row in entries:
                        file.writerow([row[1],row[2],row[4],row[6],row[8],row[10],row[11]])

                    file.writerow(["Gesamt","","","",float2duration(total),"",""])

                with open('../input/arbeitsstundenzettel.tex', 'r') as texfile:
                    texcode = texfile.read().replace('\n', '')

                texcode = texcode.replace('===NAME===', name)
                texcode = texcode.replace('===FIRSTNAME===', firstname)
                texcode = texcode.replace('===DATEOFBIRTH===', dateofbirth)
                texcode = texcode.replace('===FACULTY===', faculty)
                texcode = texcode.replace('===INSTITUTE===', institute)
                if instituteabbr in list_of_implemented_institutes:
                    texcode = texcode.replace('===INSTITUTELOGO===', '../' + logo)
                else:
                    if wanna_give_me_a_logo:
                        texcode = texcode.replace('===INSTITUTELOGO===', '../' + logo)
                    else:
                        texcode = texcode.replace('===INSTITUTELOGO===', "")
                texcode = texcode.replace('===STREET===', street)
                texcode = texcode.replace('===POSTCODE===', postcode)
                texcode = texcode.replace('===CITY===', city)
                texcode = texcode.replace('===MONTH===', monthname + ' ' + str(year))
                texcode = texcode.replace('===FILENAME===', filename + '.csv')

                with open(filename + '.tex', 'w') as texfile:
                    texfile.write(texcode)

                texcmd = ['rubber', '--pdf', filename + '.tex']
                proc = subprocess.Popen(texcmd)
                proc.communicate()

                returncode = proc.returncode
                if not returncode == 0:
                    os.unlink(filename + '.tex')
                    print("")
                    raise ValueError('Error {} executing command: {}'.format(returncode, ' '.join(texcmd)))
                else:
                    print(
'''I finished the compilation of your working hours sheet for ''' + str(year) + '-' + '{0:0>2}'.format(month) + '''.
You can find it in the pdf-folder, it is named ''' + filename + '''.pdf.'''
                    )
                    print("")

                with zipfile.ZipFile(filename + '.zip', 'w') as zippy:
                    zippy.write(filename + '.pdf')
                    zippy.write(filename + '.tex')
                    zippy.write(filename + '.csv')

                os.rename(filename + '.zip', "../output/" + filename + '.zip')

                os.unlink(filename + '.pdf')
                os.unlink(filename + '.csv')
                os.unlink(filename + '.tex')
                os.unlink(filename + '.log')
                os.unlink(filename + '.aux')

            print("")
            should_i_do_it_again = input(
'''Have a look at the result.
Want me to redo my work (no further input needed)? (yes/no)

> '''
            )
            if (should_i_do_it_again == "y" or should_i_do_it_again == "yes"):
                should_i_do_it_again = True
                advanced_settings = False
            else:
                should_i_do_it_again = False
                should_i_make_up_workinghours = False

else:

    workinghours = []

    with open(filename, 'r') as csvfile:
        file = csv.reader(
            filter(lambda row: row[0]!='#', csvfile),
            delimiter=',',
        )
        for row in file:
            row = [x.strip(' ') for x in row]
            workinghours.append(row)

    if (dt.datetime.strptime(workinghours[0][0], "%Y-%m-%d").date() - start).days < 0:
        print("Es wurden Daten angegeben, die vor dem Beginn des Arbeitsvertrages liegen!")
    elif (dt.datetime.strptime(workinghours[0][0], "%Y-%m-%d").month - start.month) > 0:
        missing_months_at_start = dt.datetime.strptime(workinghours[0][0], "%Y-%m-%d").month - start.month
        if missing_months_at_start == 1:
            print("Für den erste Monat in der Vertragslaufzeit wurden keine Arbeitszeiten angegeben!")
        else:
            print("Für die ersten " + str(missing_months_at_start) + " Monate in der Vertragslaufzeit wurden keine Arbeitszeiten angegeben!")

    if (end - dt.datetime.strptime(workinghours[-1][0], "%Y-%m-%d").date()).days < 0:
        print("Es wurden Daten angegeben, die nach dem Ende des Arbeitsvertrages liegen!")
    elif (end.month - dt.datetime.strptime(workinghours[-1][0], "%Y-%m-%d").month) > 0:
        missing_months_at_end = end.month - dt.datetime.strptime(workinghours[-1][0], "%Y-%m-%d").month
        if missing_months_at_end == 1:
            print("Für den letzten Monat in der Vertragslaufzeit wurden keine Arbeitszeiten angegeben!")
        else:
            print("Für die letzten " + str(missing_months_at_end) + " Monate in der Vertragslaufzeit wurden keine Arbeitszeiten angegeben!")

    made_up = make_up_workinghours(2, start, end, workhours_per_week, workdays_per_week)
    workinghours.append(made_up)
